<?php

declare(strict_types=1);

namespace Drupal\lb_styles_conditions\Form;

use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Layout Builder Styles Conditions settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Constructs a SettingsForm object.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typedConfigManager,
    protected ConditionManager $conditionManager,
    protected ModuleHandlerInterface $moduleHandler,
    protected ContextRepositoryInterface $contextRepository,
  ) {
    parent::__construct($config_factory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('plugin.manager.condition'),
      $container->get('module_handler'),
      $container->get('context.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'lb_styles_conditions_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['lb_styles_conditions.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('lb_styles_conditions.settings');
    $enabled_conditions = $config->get('enabled_conditions') ?: [];

    // Get all available conditions.
    $available_conditions = $this->conditionManager->getDefinitions();

    $form['conditions'] = [
      '#type' => 'details',
      '#title' => $this->t('Available Conditions'),
      '#description' => $this->t('Select which conditions should be available for Layout Builder Styles.'),
      '#open' => TRUE,
    ];

    $form['conditions']['enabled_conditions'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enabled conditions'),
      '#options' => array_map(function ($condition) {
        return $condition['label'];
      }, $available_conditions),
      '#default_value' => $enabled_conditions,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->getValues();

    // Filter out unselected conditions.
    $enabled_conditions = array_filter($values['enabled_conditions']);

    $this->config('lb_styles_conditions.settings')
      ->set('enabled_conditions', $enabled_conditions)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
